/*
 *      ical - 
 *              A very simple desk calendar.  Use the left and middle mouse
 *      mouse buttons to go forward and backward in time.
 *      
 *                              Paul Haeberli - 1985
 *
 *      modified to add alpha clock in lrh corner, yellow sun, blue moon...
 *
 *                              Seth Teller - 1989    
 *
 *	Note that Seth's version uses the B, F, and R keys to allow
 *	the user to twiddle with the sun and moon (Back, Fwd, Restore).
 *
 *	Added improvements: B and F keys increment is now 15 minutes,
 *	"Bogus Data" string displayed when B, F, or Mouse buttons change
 *	from actual time and month, current day-of-month always circled
 *	in blue, calandar reverts to current time and month 60 seconds
 *	after being twiddled.  See Readme for more info.
 *
 *				Tom Mackey - 1990
 *
 */
#include <gl.h>
#include <device.h>
#include <math.h>
#include <time.h>

#define rgb(r,g,b) cpack(((unsigned char)(b * 255.0) << 16) | \
			 ((unsigned char)(g * 255.0) << 8)  | \
			 ((unsigned char)(r * 255.0) << 0))

#define grey(i) cpack(((unsigned char)(i * 255.0) << 16) | \
		      ((unsigned char)(i * 255.0) << 8)  | \
		      ((unsigned char)(i * 255.0) << 0))

#define PI (3.1415926)

char *monthnames[] = {  "January", "February", "March", "April",
			"May", "June", "July", "August",
			"September", "October", "November", "December" };

char *daynames[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

char monlength[] = { 0,
		    31, 29, 31, 30,
		    31, 30, 31, 31,
		    30, 31, 30, 31 };

struct  caltime
{
    int second;
    int minute;
    int hour;
    int day;
    int month;
    int year;
};

struct caltime Today = { -1, -1, -1, -1, -1, -1 }, Show;
int CalWin = 0;
long Toffset = 0;
int BogusTime = 0;

void print_timeofday();

void update_time (show, today)
struct caltime *show, *today;
{
    static long lasttime, thetime, showtime, lastreq;
    static float correction = 1.0;
    static struct tm *tm, *showtm;

    lasttime = thetime;
    thetime = time(0);
    showtime = thetime + Toffset;

    tm = localtime(&thetime);

    today->second = tm->tm_sec;
    if (today->minute != tm->tm_min) {
        today->minute = tm->tm_min;
        qenter (REDRAW, CalWin);
    }
    today->hour = tm->tm_hour;

    today->year = tm->tm_year + 1900;
    today->month = tm->tm_mon + 1;
    today->day = tm->tm_mday;

    showtm = localtime(&showtime);

    show->day = showtm->tm_mday;
    show->second = showtm->tm_sec;
    show->minute = showtm->tm_min;
    show->hour = showtm->tm_hour;

    if (lastreq && thetime - lasttime >= 30 && !Toffset)
        correction = (lastreq / 60.0) / (thetime - lasttime);

    lastreq = (int)(20 + 60 * (60 - today->second) * correction);

    if (!Toffset)
        noise (TIMER0, lastreq);
    else
        noise (TIMER0, 3600);
}

main(argc, argv)
char *argv[];
{
    short val;
    static long deadline;

    prefsize(260,185);

    CalWin = winopen("TurboCal");
    if (getplanes() > 8)
        doublebuffer();
    RGBmode();
    gconfig();

    qdevice(REDRAW);
    qdevice(LEFTMOUSE);
    qdevice(MIDDLEMOUSE);
    qdevice(TIMER0);
    qdevice(BKEY);
    qdevice(FKEY);
    qdevice(RKEY);

    update_time(&Show, &Today);

    if (argc >= 2) {
        Show.month = atoi(argv[1]);
        Show.year = atoi(argv[2]);
        Show.day = 1;
    } else
        Show = Today;

    while (1) {
        switch (qread(&val)) {

            case BKEY :
                if (val)    {
                    Toffset -= 15 * 60;  /* 15 minutes */
		    deadline = time(0);
		    BogusTime = 1;
                    update_time(&Show, &Today);
                    drawcal(&Show, &Today);
		}
                break;

            case FKEY :
                if (val)    {
                    Toffset += 15 * 60;  /* 15 minutes */
		    deadline = time(0);
		    BogusTime = 1;
                    update_time(&Show, &Today);
                    drawcal(&Show, &Today);
		}
                break;

            case RKEY :
                if (val)    {
                    Toffset = 0;
                    update_time(&Show, &Today);
                    drawcal(&Show, &Today);
		}
                break;

            case LEFTMOUSE :
                if (val) {
                    Show.month--;
                    if (Show.month == 0) {
                        Show.year--;
                        Show.month = 12;
                    }
		    deadline = time(0);
		    BogusTime = 1;
                    drawcal(&Show, &Today);
                }
                break;

            case MIDDLEMOUSE :
                if (val) {
                    Show.month++;
                    if (Show.month == 13) {
                        Show.year++;
                        Show.month = 1;
		    }
		    deadline = time(0);
		    BogusTime = 1;
		    drawcal(&Show, &Today);
		}
                break;

            case REDRAW :
                drawcal(&Show, &Today);
                break;

	    case TIMER0 :
		if ((time(0) - deadline) > 20 && BogusTime)
		{
		    Show.month = Today.month;
		    Show.year = Today.year;
		    BogusTime = FALSE;
		    qenter(RKEY, 1);
		}
		else
		    update_time(&Show, &Today);
		break;

	    default:
		break;
	    }
    }
}

int drawcal(show, today)
struct caltime *show, *today;
{
    register i;
    int c;
    int ycoord;
    int samemonth;
    char tempstr[30];

    reshapeviewport();
    gsync();

    /* find out if we are showing the current month */
    samemonth = ((today->year == show->year) && (today->month == show->month));

    grey(0.8);
    clear();
    grey(0.0);

    {
	/* draw in the sun and the moon */
	/* sun as yellow circle, moon as blue crescent */

	static short left, right, bottom, top;
	int sunx, suny;
	float sung;			    /* green comp */

	if (left == right)
	    getscrmask (&left, &right, &bottom, &top); 

	/* scr mask to clip to top half of screen */
	scrmask (left, right, bottom + (top - bottom) / 2 + 8, top);

	/* draw sun */
	sunx = 130.0 - (60.0 * sin (2.0 * PI * ((show->hour / 24.0) +
		show->minute/(60.0 * 24.0)))); 

	suny =  98.0 - (60.0 * cos (2.0 * PI * ((show->hour / 24.0) +
		show->minute/(60.0 * 24.0)))); 

	if (suny < 118.0 && suny >= 98.0)
	    sung = (12.8 * (suny - 98.0)) / 255.0;  
	else if (suny < 98.0 && suny >= 78.0)
	    sung = 0.0;
	else
	    sung = 1.0;

	rgb (1.0, sung, 0.0);
	circf (sunx, suny, 20.0);

	/* scr mask to clip to bottom half of screen */
	scrmask (left, right, bottom, bottom + (top - bottom) / 2 - 1 + 8);

	rgb (0.0, 0.0, 1.0);
	circfi ((int)sunx, (int)suny, 20);
	grey (0.8);
	circfi ((int)sunx - 10, (int)suny, 20);

	/* scr mask back to normal */
	scrmask (left, right, bottom, top);

	rgb(0.0, 1.0, 0.0);
	move2i(20,99);
	draw2i(240,99);
    }

    grey(0.5);
    move2i(20,163);
    draw2i(240,163);

    rgb(0.0,0.0,1.0);
    cmov2i(20,165);
    charstr(monthnames[show->month-1]);
    rgb(1.0,0.0,0.0);
    sprintf(tempstr,"%u",show->year);
    cmov2i(208,165);
    charstr(tempstr);

    rgb(0.5, 0.5, 0.5);
    move2i(20,143);
    draw2i(240,143);

    grey(0.0);

    for (i=0; i<7; i++){
        cmov2i(24+32*i,145);
        charstr(daynames[i]);
    }

    c = dayofweek(show->month, show->year);
    ycoord = 125;
    for (i=1; i<=monlength[show->month]; i++) {
        cmov2i(24+32*c,ycoord);
        sprintf(tempstr,"%2d",i); 
        if ((i == today->day) && samemonth) {
	    pushattributes();
		rgb(1.0,0.0,0.0);
		charstr(tempstr);
	    popattributes();
	}
	else {
	    charstr(tempstr);
	}

	if (i == show->day) {
	    pushattributes();
		rgb(0.0,0.0,1.0);
		circi((24+32*c)+8, ycoord+5, 12);
	    popattributes();
	}

	/* if finished with this line, drop down to next */
        if (++c == 7) {
            c = 0;
            ycoord -= 20;
        }
    }

    if (Toffset)
	print_timeofday(show);
    else
	print_timeofday(today);

    if (BogusTime)
    {
	pushattributes();
	    rgb(0.15,0.5,0.0);
	    cmov2i(20,10);
	    charstr("Bogus Data");
	popattributes();
    }

    swapbuffers();
}

/*
** build and print time of day string
*/
void print_timeofday(someday)
struct caltime *someday;
{
    char tempstr[30];

    pushattributes();
	rgb(0.0,0.0,1.0);
	cmov2i(176, 10);
	sprintf(tempstr,"%2d:%02d %cm", (someday->hour - 1 + 12) % 12 + 1,
			    someday->minute, someday->hour > 11 ? 'p' : 'a'); 
	charstr(tempstr);
    popattributes();
}

dayofweek(m, y)
{
    register d, i;

    d = jan1(y);

    if (((jan1(y+1)+7-d)%7) == 1)
        monlength[2] = 28;
    else
        monlength[2] = 29;

    for (i=1; i<m; i++)
        d += monlength[i];

    return d%7;
}

/*
 *      return day of the week
 *      of jan 1 of given year
 */
jan1(y)
int y;
{
    register int d;

    d = y + 4 + (y+3)/4;

    if (y > 1752) {
        d += 3;
        if (y > 1800) {
            d -= (y-1701)/100;
            d += (y-1601)/400;
        }
    }
    return d%7;
}
